@@ -1,6 +1,6 @@
 /*
  * ElementTree
- * $Id: cElementTree.c 2322 2005-03-13 19:37:10Z fredrik $
+ * $Id: /work/modules/celementtree/cElementTree.c 1128 2005-12-16T21:57:13.668520Z Fredrik  $
  *
  * elementtree accelerator
  *
@@ -27,17 +27,23 @@
  * 2005-01-28 fl  added remove method (1.0.1)
  * 2005-03-01 fl  added iselement function; fixed makeelement aliasing (1.0.2)
  * 2005-03-13 fl  export Comment and ProcessingInstruction/PI helpers
+ * 2005-03-26 fl  added Comment and PI support to XMLParser
+ * 2005-03-27 fl  event optimizations; complain about bogus events
+ * 2005-08-08 fl  fixed read error handling in parse
+ * 2005-08-11 fl  added runtime test for copy workaround (1.0.3)
+ * 2005-12-13 fl  added expat_capi support (for xml.etree) (1.0.4)
+ * 2005-12-16 fl  added support for non-standard encodings
  *
  * Copyright (c) 1999-2005 by Secret Labs AB.  All rights reserved.
  * Copyright (c) 1999-2005 by Fredrik Lundh.
- * 
+ *
  * info@pythonware.com
  * http://www.pythonware.com
  */
 
 #include "Python.h"
 
-#define VERSION "1.0.3"
+#define VERSION "1.0.5"
 
 /* -------------------------------------------------------------------- */
 /* configuration */
@@ -45,6 +51,9 @@
 /* Leave defined to include the expat-based XMLParser type */
 #define USE_EXPAT
 
+/* Define to to all expat calls via pyexpat's embedded expat library */
+/* #define USE_PYEXPAT_CAPI */
+
 /* An element can hold this many children without extra memory
    allocations. */
 #define STATIC_CHILDREN 4
@@ -91,6 +100,12 @@ do { memory -= size; printf("%8d - %s\n", memory, comment); } while (0)
 #endif
 #endif
 
+#if (PY_VERSION_HEX >= 0x02050000)
+#define PY_CONST const /* 2.5 adds const to some API:s */
+#else
+#define PY_CONST
+#endif
+
 #if !defined(Py_RETURN_NONE)
 #define Py_RETURN_NONE return Py_INCREF(Py_None), Py_None
 #endif
@@ -1037,6 +1052,12 @@ element_reduce(ElementObject* self, PyObject* args)
     if (!PyArg_ParseTuple(args, ":__reduce__"))
         return NULL;
 
+    /* Hack alert: This method is used to work around a __copy__
+       problem on certain 2.3 and 2.4 versions.  To save time and
+       simplify the code, we create the copy in here, and use a dummy
+       copyelement helper to trick the copy module into doing the
+       right thing. */
+
     if (!elementtree_copyelement_obj) {
         PyErr_SetString(
             PyExc_RuntimeError,
@@ -1045,12 +1066,6 @@ element_reduce(ElementObject* self, PyObject* args)
         return NULL;
     }
 
-    /* Python 2.3 and later no longer understands the __copy__ method
-       on C objects, so we have to fake it using __reduce__.  to save
-       time and simplify the code, we create the copy in here, and use
-       a dummy copyelement helper to trick the copy module into doing
-       the right thing */
-
     return Py_BuildValue(
         "O(N)", elementtree_copyelement_obj, element_copy(self, args)
         );
@@ -1074,10 +1089,12 @@ element_remove(ElementObject* self, PyObject* args)
         return NULL;
     }
 
-    /* FIXME: use explicit compare instead? */
-    for (i = 0; i < self->extra->length; i++)
+    for (i = 0; i < self->extra->length; i++) {
+        if (self->extra->children[i] == element)
+            break;
         if (PyObject_Compare(self->extra->children[i], element) == 0)
             break;
+    }
 
     if (i == self->extra->length) {
         /* element is not in children, so raise exception */
@@ -1237,23 +1254,43 @@ element_setitem(ElementObject* self, int index, PyObject* item)
 }
 
 static PyMethodDef element_methods[] = {
+
     {"clear", (PyCFunction) element_clear, METH_VARARGS},
+
     {"get", (PyCFunction) element_get, METH_VARARGS},
     {"set", (PyCFunction) element_set, METH_VARARGS},
+
     {"find", (PyCFunction) element_find, METH_VARARGS},
     {"findtext", (PyCFunction) element_findtext, METH_VARARGS},
     {"findall", (PyCFunction) element_findall, METH_VARARGS},
+
     {"append", (PyCFunction) element_append, METH_VARARGS},
     {"insert", (PyCFunction) element_insert, METH_VARARGS},
     {"remove", (PyCFunction) element_remove, METH_VARARGS},
+
     {"getiterator", (PyCFunction) element_getiterator, METH_VARARGS},
     {"getchildren", (PyCFunction) element_getchildren, METH_VARARGS},
+
     {"items", (PyCFunction) element_items, METH_VARARGS},
     {"keys", (PyCFunction) element_keys, METH_VARARGS},
+
     {"makeelement", (PyCFunction) element_makeelement, METH_VARARGS},
+
     {"__copy__", (PyCFunction) element_copy, METH_VARARGS},
     {"__deepcopy__", (PyCFunction) element_deepcopy, METH_VARARGS},
-    {"__reduce__", (PyCFunction) element_reduce, METH_VARARGS},
+
+    /* Some 2.3 and 2.4 versions do not handle the __copy__ method on
+       C objects correctly, so we have to fake it using a __reduce__-
+       based hack (see the element_reduce implementation above for
+       details). */
+
+    /* The behaviour has been changed in 2.3.5 and 2.4.1, so we're
+       using a runtime test to figure out if we need to fake things
+       or now (see the init code below).  The following entry is
+       enabled only if the hack is needed. */
+
+    {"!__reduce__", (PyCFunction) element_reduce, METH_VARARGS},
+
     {NULL, NULL}
 };
 
@@ -1628,6 +1665,7 @@ treebuilder_handle_namespace(TreeBuilderObject* self, int start,
         if (!self->start_ns_event_obj)
             return;
         action = self->start_ns_event_obj;
+        /* FIXME: prefix and uri use utf-8 encoding! */
         parcel = Py_BuildValue("ss", (prefix) ? prefix : "", uri);
         if (!parcel)
             return;
@@ -1700,74 +1738,6 @@ treebuilder_close(TreeBuilderObject* self, PyObject* args)
     return treebuilder_done(self);
 }
 
-static PyObject*
-treebuilder_setevents(TreeBuilderObject* self, PyObject* args)
-{
-    /* activate element event reporting */
-
-    int i;
-
-    PyObject* events; /* event collector */
-    PyObject* event_set = Py_None;
-    if (!PyArg_ParseTuple(args, "O!|O:_setevents",  &PyList_Type, &events,
-                          &event_set))
-        return NULL;
-
-    Py_INCREF(events);
-    Py_XDECREF(self->events);
-    self->events = events;
-
-    /* clear out existing events */
-    Py_XDECREF(self->start_event_obj); self->start_event_obj = NULL;
-    Py_XDECREF(self->end_event_obj); self->end_event_obj = NULL;
-    Py_XDECREF(self->start_ns_event_obj); self->start_ns_event_obj = NULL;
-    Py_XDECREF(self->end_ns_event_obj); self->end_ns_event_obj = NULL;
-
-    if (event_set == Py_None) {
-        /* default is "end" only */
-        self->end_event_obj = PyString_FromString("end");
-        Py_RETURN_NONE;
-    }
-
-    if (!PyTuple_Check(event_set)) /* FIXME: handle arbitrary sequences */
-        goto error;
-
-    for (i = 0; i < PyTuple_GET_SIZE(event_set); i++) {
-        PyObject* item = PyTuple_GET_ITEM(event_set, i);
-        char* event;
-        if (!PyString_Check(item))
-            goto error;
-        event = PyString_AS_STRING(item);
-        if (strcmp(event, "start") == 0) {
-            Py_INCREF(item);
-            self->start_event_obj = item;
-        } else if (strcmp(event, "end") == 0) {
-            Py_INCREF(item);
-            Py_XDECREF(self->end_event_obj);
-            self->end_event_obj = item;
-        } else if (strcmp(event, "start-ns") == 0) {
-            Py_INCREF(item);
-            Py_XDECREF(self->start_ns_event_obj);
-            self->start_ns_event_obj = item;
-        } else if (strcmp(event, "end-ns") == 0) {
-            Py_INCREF(item);
-            Py_XDECREF(self->end_ns_event_obj);
-            self->end_ns_event_obj = item;
-        } else
-            goto error;
-    }
-
-    Py_RETURN_NONE;
-
-  error:
-    PyErr_SetString(
-        PyExc_TypeError,
-        "invalid event tuple"
-        );
-    return NULL;
-    
-}
-
 static PyObject*
 treebuilder_start(TreeBuilderObject* self, PyObject* args)
 {
@@ -1796,7 +1766,6 @@ static PyMethodDef treebuilder_methods[] = {
     {"end", (PyCFunction) treebuilder_end, METH_VARARGS},
     {"xml", (PyCFunction) treebuilder_xml, METH_VARARGS},
     {"close", (PyCFunction) treebuilder_close, METH_VARARGS},
-    {"_setevents", (PyCFunction) treebuilder_setevents, METH_VARARGS},
     {NULL, NULL}
 };
 
@@ -1822,6 +1791,14 @@ statichere PyTypeObject TreeBuilder_Type = {
 
 #include "expat.h"
 
+#if defined(USE_PYEXPAT_CAPI)
+#include "pyexpat.h"
+static struct PyExpat_CAPI* expat_capi;
+#define EXPAT(func) (expat_capi->func)
+#else
+#define EXPAT(func) (XML_##func)
+#endif
+
 typedef struct {
     PyObject_HEAD
 
@@ -1837,6 +1814,9 @@ typedef struct {
     PyObject* handle_data;
     PyObject* handle_end;
 
+    PyObject* handle_comment;
+    PyObject* handle_pi;
+
 } XMLParserObject;
 
 staticforward PyTypeObject XMLParser_Type;
@@ -1979,8 +1959,8 @@ expat_default_handler(XMLParserObject* self, const XML_Char* data_in,
         PyErr_Format(
             PyExc_SyntaxError, "undefined entity &%s;: line %d, column %d",
             PyString_AS_STRING(key),
-            XML_GetErrorLineNumber(self->parser),
-            XML_GetErrorColumnNumber(self->parser)
+            EXPAT(GetErrorLineNumber)(self->parser),
+            EXPAT(GetErrorColumnNumber)(self->parser)
             );
     }
 
@@ -2106,6 +2086,84 @@ expat_end_ns_handler(XMLParserObject* self, const XML_Char* prefix_in)
         );
 }
 
+static void
+expat_comment_handler(XMLParserObject* self, const XML_Char* comment_in)
+{
+    PyObject* comment;
+    PyObject* res;
+
+    if (self->handle_comment) {
+        comment = makestring(comment_in, strlen(comment_in));
+        if (comment) {
+            res = PyObject_CallFunction(self->handle_comment, "O", comment);
+            Py_XDECREF(res);
+            Py_DECREF(comment);
+        }
+    }
+}
+
+static void
+expat_pi_handler(XMLParserObject* self, const XML_Char* target_in,
+                 const XML_Char* data_in)
+{
+    PyObject* target;
+    PyObject* data;
+    PyObject* res;
+
+    if (self->handle_pi) {
+        target = makestring(target_in, strlen(target_in));
+        data = makestring(data_in, strlen(data_in));
+        if (target && data) {
+            res = PyObject_CallFunction(self->handle_pi, "OO", target, data);
+            Py_XDECREF(res);
+            Py_DECREF(data);
+            Py_DECREF(target);
+        } else {
+            Py_XDECREF(data);
+            Py_XDECREF(target);
+        }
+    }
+}
+
+#if defined(Py_USING_UNICODE)
+static int
+expat_unknown_encoding_handler(XMLParserObject *self, const XML_Char *name,
+                               XML_Encoding *info)
+{
+    PyObject* u;
+    Py_UNICODE* p;
+    unsigned char s[256];
+    int i;
+
+    memset(info, 0, sizeof(XML_Encoding));
+
+    for (i = 0; i < 256; i++)
+        s[i] = i;
+    
+    u = PyUnicode_Decode(s, 256, name, "replace");
+    if (!u)
+        return XML_STATUS_ERROR;
+
+    if (PyUnicode_GET_SIZE(u) != 256) {
+        Py_DECREF(u);
+        return XML_STATUS_ERROR;
+    }
+
+    p = PyUnicode_AS_UNICODE(u);
+
+    for (i = 0; i < 256; i++) {
+	if (p[i] != Py_UNICODE_REPLACEMENT_CHARACTER)
+	    info->map[i] = p[i];
+        else
+	    info->map[i] = -1;
+    }
+
+    Py_DECREF(u);
+
+    return XML_STATUS_OK;
+}
+#endif
+
 /* -------------------------------------------------------------------- */
 /* constructor and destructor */
 
@@ -2118,11 +2176,20 @@ xmlparser(PyObject* _self, PyObject* args, PyObject* kw)
 
     PyObject* target = NULL;
     char* encoding = NULL;
-    static char* kwlist[] = { "target", "encoding", NULL };
+    static PY_CONST char* kwlist[] = { "target", "encoding", NULL };
     if (!PyArg_ParseTupleAndKeywords(args, kw, "|Oz:XMLParser", kwlist,
                                      &target, &encoding))
         return NULL;
 
+#if defined(USE_PYEXPAT_CAPI)
+    if (!expat_capi) {
+        PyErr_SetString(
+            PyExc_RuntimeError, "cannot load dispatch table from pyexpat"
+            );
+        return NULL;
+    }
+#endif
+
     self = PyObject_New(XMLParserObject, &XMLParser_Type);
     if (self == NULL)
         return NULL;
@@ -2143,7 +2210,7 @@ xmlparser(PyObject* _self, PyObject* args, PyObject* kw)
     memory_handler.realloc_fcn = PyObject_Realloc;
     memory_handler.free_fcn = PyObject_Free;
 
-    self->parser = XML_ParserCreate_MM(encoding, &memory_handler, "}");
+    self->parser = EXPAT(ParserCreate_MM)(encoding, &memory_handler, "}");
     if (!self->parser) {
         PyErr_NoMemory();
         return NULL; /* FIXME: cleanup on error */
@@ -2164,32 +2231,42 @@ xmlparser(PyObject* _self, PyObject* args, PyObject* kw)
     self->handle_start = PyObject_GetAttrString(target, "start");
     self->handle_data = PyObject_GetAttrString(target, "data");
     self->handle_end = PyObject_GetAttrString(target, "end");
+    self->handle_comment = PyObject_GetAttrString(target, "comment");
+    self->handle_pi = PyObject_GetAttrString(target, "pi");
 
     PyErr_Clear();
 
     /* configure parser */
-    XML_SetUserData(self->parser, self);
-    XML_SetElementHandler(
+    EXPAT(SetUserData)(self->parser, self);
+    EXPAT(SetElementHandler)(
         self->parser,
         (XML_StartElementHandler) expat_start_handler,
         (XML_EndElementHandler) expat_end_handler
         );
-    XML_SetDefaultHandlerExpand(
+    EXPAT(SetDefaultHandlerExpand)(
         self->parser,
         (XML_DefaultHandler) expat_default_handler
         );
-    XML_SetCharacterDataHandler(
+    EXPAT(SetCharacterDataHandler)(
         self->parser,
         (XML_CharacterDataHandler) expat_data_handler
         );
-
-    if (TreeBuilder_CheckExact(target))
-        /* FIXME: use option to enable this */
-        XML_SetNamespaceDeclHandler(
+    if (self->handle_comment)
+        EXPAT(SetCommentHandler)(
             self->parser,
-            (XML_StartNamespaceDeclHandler) expat_start_ns_handler,
-            (XML_EndNamespaceDeclHandler) expat_end_ns_handler
+            (XML_CommentHandler) expat_comment_handler
             );
+    if (self->handle_pi)
+        EXPAT(SetProcessingInstructionHandler)(
+            self->parser,
+            (XML_ProcessingInstructionHandler) expat_pi_handler
+            );
+#if defined(Py_USING_UNICODE)
+    EXPAT(SetUnknownEncodingHandler)(
+        self->parser,
+        (XML_UnknownEncodingHandler) expat_unknown_encoding_handler, NULL
+        );
+#endif
 
     ALLOC(sizeof(XMLParserObject), "create expatparser");
 
@@ -2199,8 +2276,10 @@ xmlparser(PyObject* _self, PyObject* args, PyObject* kw)
 static void
 xmlparser_dealloc(XMLParserObject* self)
 {
-    XML_ParserFree(self->parser);
+    EXPAT(ParserFree)(self->parser);
 
+    Py_XDECREF(self->handle_pi);
+    Py_XDECREF(self->handle_comment);
     Py_XDECREF(self->handle_end);
     Py_XDECREF(self->handle_data);
     Py_XDECREF(self->handle_start);
@@ -2223,7 +2302,7 @@ expat_parse(XMLParserObject* self, char* data, int data_len, int final)
 {
     int ok;
 
-    ok = XML_Parse(self->parser, data, data_len, final);
+    ok = EXPAT(Parse)(self->parser, data, data_len, final);
 
     if (PyErr_Occurred())
         return NULL;
@@ -2231,9 +2310,9 @@ expat_parse(XMLParserObject* self, char* data, int data_len, int final)
     if (!ok) {
         PyErr_Format(
             PyExc_SyntaxError, "%s: line %d, column %d",
-            XML_ErrorString(XML_GetErrorCode(self->parser)),
-            XML_GetErrorLineNumber(self->parser),
-            XML_GetErrorColumnNumber(self->parser)
+            EXPAT(ErrorString)(EXPAT(GetErrorCode)(self->parser)),
+            EXPAT(GetErrorLineNumber)(self->parser),
+            EXPAT(GetErrorColumnNumber)(self->parser)
             );
         return NULL;
     }
@@ -2294,6 +2373,13 @@ xmlparser_parse(XMLParserObject* self, PyObject* args)
     for (;;) {
 
         buffer = PyObject_CallFunction(reader, "i", 64*1024);
+
+        if (!buffer) {
+            /* read failed (e.g. due to KeyboardInterrupt) */
+            Py_DECREF(reader);
+            return NULL;
+        }
+
         if (!PyString_CheckExact(buffer) || PyString_GET_SIZE(buffer) == 0) {
             Py_DECREF(buffer);
             break;
@@ -2325,10 +2411,105 @@ xmlparser_parse(XMLParserObject* self, PyObject* args)
     return res;
 }
 
+static PyObject*
+xmlparser_setevents(XMLParserObject* self, PyObject* args)
+{
+    /* activate element event reporting */
+
+    int i;
+    TreeBuilderObject* target;
+
+    PyObject* events; /* event collector */
+    PyObject* event_set = Py_None;
+    if (!PyArg_ParseTuple(args, "O!|O:_setevents",  &PyList_Type, &events,
+                          &event_set))
+        return NULL;
+
+    if (!TreeBuilder_CheckExact(self->target)) {
+        PyErr_SetString(
+            PyExc_TypeError,
+            "event handling only supported for cElementTree.Treebuilder "
+            "targets"
+            );
+        return NULL;
+    }
+
+    target = (TreeBuilderObject*) self->target;
+
+    Py_INCREF(events);
+    Py_XDECREF(target->events);
+    target->events = events;
+
+    /* clear out existing events */
+    Py_XDECREF(target->start_event_obj); target->start_event_obj = NULL;
+    Py_XDECREF(target->end_event_obj); target->end_event_obj = NULL;
+    Py_XDECREF(target->start_ns_event_obj); target->start_ns_event_obj = NULL;
+    Py_XDECREF(target->end_ns_event_obj); target->end_ns_event_obj = NULL;
+
+    if (event_set == Py_None) {
+        /* default is "end" only */
+        target->end_event_obj = PyString_FromString("end");
+        Py_RETURN_NONE;
+    }
+
+    if (!PyTuple_Check(event_set)) /* FIXME: handle arbitrary sequences */
+        goto error;
+
+    for (i = 0; i < PyTuple_GET_SIZE(event_set); i++) {
+        PyObject* item = PyTuple_GET_ITEM(event_set, i);
+        char* event;
+        if (!PyString_Check(item))
+            goto error;
+        event = PyString_AS_STRING(item);
+        if (strcmp(event, "start") == 0) {
+            Py_INCREF(item);
+            target->start_event_obj = item;
+        } else if (strcmp(event, "end") == 0) {
+            Py_INCREF(item);
+            Py_XDECREF(target->end_event_obj);
+            target->end_event_obj = item;
+        } else if (strcmp(event, "start-ns") == 0) {
+            Py_INCREF(item);
+            Py_XDECREF(target->start_ns_event_obj);
+            target->start_ns_event_obj = item;
+            EXPAT(SetNamespaceDeclHandler)(
+                self->parser,
+                (XML_StartNamespaceDeclHandler) expat_start_ns_handler,
+                (XML_EndNamespaceDeclHandler) expat_end_ns_handler
+                );
+        } else if (strcmp(event, "end-ns") == 0) {
+            Py_INCREF(item);
+            Py_XDECREF(target->end_ns_event_obj);
+            target->end_ns_event_obj = item;
+            EXPAT(SetNamespaceDeclHandler)(
+                self->parser,
+                (XML_StartNamespaceDeclHandler) expat_start_ns_handler,
+                (XML_EndNamespaceDeclHandler) expat_end_ns_handler
+                );
+        } else {
+            PyErr_Format(
+                PyExc_ValueError,
+                "unknown event '%s'", event
+                );
+            return NULL;
+        }
+    }
+
+    Py_RETURN_NONE;
+
+  error:
+    PyErr_SetString(
+        PyExc_TypeError,
+        "invalid event tuple"
+        );
+    return NULL;
+}
+
 static PyMethodDef xmlparser_methods[] = {
     {"feed", (PyCFunction) xmlparser_feed, METH_VARARGS},
     {"close", (PyCFunction) xmlparser_close, METH_VARARGS},
     {"_parse", (PyCFunction) xmlparser_parse, METH_VARARGS},
+    {"_setevents", (PyCFunction) xmlparser_setevents, METH_VARARGS},
     {NULL, NULL}
 };
 
@@ -2391,6 +2572,10 @@ initcElementTree(void)
 {
     PyObject* m;
     PyObject* g;
+    char* bootstrap;
+#if defined(USE_PYEXPAT_CAPI)
+    struct PyExpat_CAPI* capi;
+#endif
 
     /* Patch object type */
     Element_Type.ob_type = TreeBuilder_Type.ob_type = &PyType_Type;
@@ -2406,23 +2591,28 @@ initcElementTree(void)
 
     PyDict_SetItemString(g, "__builtins__", PyEval_GetBuiltins());
 
-    PyRun_String(
+    bootstrap = (
 
 #if (PY_VERSION_HEX >= 0x02020000 && PY_VERSION_HEX < 0x02030000)
         "from __future__ import generators\n" /* enable yield under 2.2 */
 #endif
 
-        "from copy import deepcopy\n"
+        "from copy import copy, deepcopy\n"
+
         "try:\n"
         "  from elementtree import ElementTree\n"
         "except ImportError:\n"
         "  import ElementTree\n"
         "ET = ElementTree\n"
         "del ElementTree\n"
+
         "import cElementTree\n"
 
-        "def copyelement(elem):\n" /* helper */
-        "  return elem\n"
+        "try:\n" /* check if copy works as is */
+        "  copy(cElementTree.Element('x'))\n"
+        "except:\n"
+        "  def copyelement(elem):\n"
+        "    return elem\n"
 
         "def Comment(text=None):\n" /* public */
         "  element = cElementTree.Element(ET.Comment)\n"
@@ -2472,7 +2662,8 @@ initcElementTree(void)
         "cElementTree.parse = parse\n"
 
 #if (PY_VERSION_HEX < 0x02020000)
-        /* FIXME: iterparse not yet available for 2.1 */
+        "if hasattr(ET, 'iterparse'):\n"
+        "    cElementTree.iterparse = ET.iterparse\n" /* delegate on 2.1 */
 #else
         "class iterparse(object):\n"
         " root = None\n"
@@ -2484,8 +2675,8 @@ initcElementTree(void)
         " def __iter__(self):\n" 
         "  events = []\n"
         "  b = cElementTree.TreeBuilder()\n"
-        "  b._setevents(events, self._events)\n"
         "  p = cElementTree.XMLParser(b)\n"
+        "  p._setevents(events, self._events)\n"
         "  while 1:\n"
         "    data = self._file.read(16384)\n"
         "    if not data:\n"
@@ -2538,12 +2729,38 @@ initcElementTree(void)
         "cElementTree.__version__ = '" VERSION "'\n"
         "cElementTree.XMLParserError = SyntaxError\n"
 
-        "", Py_file_input, g, NULL
-        );
+       );
+
+    PyRun_String(bootstrap, Py_file_input, g, NULL);
 
     elementpath_obj = PyDict_GetItemString(g, "ElementPath");
 
     elementtree_copyelement_obj = PyDict_GetItemString(g, "copyelement");
+    if (elementtree_copyelement_obj) {
+        /* reduce hack needed; enable reduce method */
+        PyMethodDef* mp;
+        for (mp = element_methods; mp->ml_name; mp++)
+            if (mp->ml_meth == (PyCFunction) element_reduce) {
+                mp->ml_name = "__reduce__";
+                break;
+            }
+    } else
+        PyErr_Clear();
     elementtree_deepcopy_obj = PyDict_GetItemString(g, "deepcopy");
     elementtree_getiterator_obj = PyDict_GetItemString(g, "getiterator");
+
+#if defined(USE_PYEXPAT_CAPI)
+    /* link against pyexpat, if possible */
+    capi = PyCObject_Import("pyexpat", "expat_CAPI");
+    if (capi &&
+        strcmp(capi->magic, PyExpat_CAPI_MAGIC) == 0 &&
+        capi->size <= sizeof(*expat_capi) &&
+        capi->MAJOR_VERSION == XML_MAJOR_VERSION &&
+        capi->MINOR_VERSION == XML_MINOR_VERSION &&
+        capi->MICRO_VERSION == XML_MICRO_VERSION)
+        expat_capi = capi;
+    else
+        expat_capi = NULL;
+#endif
+
 }
