@@ -1,4 +1,4 @@
-# $Id: selftest.py 2322 2005-03-13 19:37:10Z fredrik $
+# $Id: /work/modules/celementtree/selftest.py 1128 2005-12-16T21:57:13.668520Z Fredrik  $
 # elementtree selftest program
 
 # this test script uses Python's "doctest" module to check that the
@@ -99,9 +99,9 @@ def check_method(method):
 def version():
     """
     >>> ElementTree.__version__
-    '1.0.3'
+    '1.0.5'
     >>> ElementTree.VERSION
-    '1.0.3'
+    '1.0.5'
     """
 
 def element():
@@ -466,9 +466,7 @@ def iterparse():
     """
     Test iterparse interface.
 
-    >>> iterparse = ElementTree.iterparse
-
-    >>> context = iterparse("samples/simple.xml")
+    >>> context = ElementTree.iterparse("samples/simple.xml")
     >>> for action, elem in context:
     ...   print action, elem.tag
     end element
@@ -478,7 +476,7 @@ def iterparse():
     >>> context.root.tag
     'root'
 
-    >>> context = iterparse("samples/simple-ns.xml")
+    >>> context = ElementTree.iterparse("samples/simple-ns.xml")
     >>> for action, elem in context:
     ...   print action, elem.tag
     end {namespace}element
@@ -487,17 +485,17 @@ def iterparse():
     end {namespace}root
 
     >>> events = ()
-    >>> context = iterparse("samples/simple.xml", events)
+    >>> context = ElementTree.iterparse("samples/simple.xml", events)
     >>> for action, elem in context:
     ...   print action, elem.tag
 
     >>> events = ()
-    >>> context = iterparse("samples/simple.xml", events=events)
+    >>> context = ElementTree.iterparse("samples/simple.xml", events=events)
     >>> for action, elem in context:
     ...   print action, elem.tag
 
     >>> events = ("start", "end")
-    >>> context = iterparse("samples/simple.xml", events)
+    >>> context = ElementTree.iterparse("samples/simple.xml", events)
     >>> for action, elem in context:
     ...   print action, elem.tag
     start root
@@ -509,14 +507,13 @@ def iterparse():
     end empty-element
     end root
 
-    >>> events = ("start", "end", "start-ns", "end-ns")
-    >>> context = iterparse("samples/simple-ns.xml", events)
+    >>> events = ("start", "end")
+    >>> context = ElementTree.iterparse("samples/simple-ns.xml", events)
     >>> for action, elem in context:
     ...   if action in ("start", "end"):
     ...     print action, elem.tag
     ...   else:
     ...     print action, elem
-    start-ns ('', 'namespace')
     start {namespace}root
     start {namespace}element
     end {namespace}element
@@ -525,8 +522,19 @@ def iterparse():
     start {namespace}empty-element
     end {namespace}empty-element
     end {namespace}root
-    end-ns None
 
+    >>> events = ("start", "end", "bogus")
+    >>> context = ElementTree.iterparse("samples/simple.xml", events)
+    >>> if sys.version[:3] > "2.1": # don't apply this test for 2.1
+    ...   for action, elem in context:
+    ...     if action in ("start", "end"):
+    ...       print action, elem.tag
+    ...     else:
+    ...       print action, elem
+    ... else:
+    ...   raise ValueError("unknown event 'bogus'")
+    Traceback (most recent call last):
+    ValueError: unknown event 'bogus'
     """
 
 def custom_builder():
@@ -552,6 +560,31 @@ def custom_builder():
     end empty-element
     end root
 
+    >>> class Builder:
+    ...     def start(self, tag, attrib):
+    ...         print "start", tag
+    ...     def end(self, tag):
+    ...         print "end", tag
+    ...     def data(self, text):
+    ...         pass
+    ...     def pi(self, target, data):
+    ...         print "pi", target, repr(data)
+    ...     def comment(self, data):
+    ...         print "comment", repr(data)
+    >>> builder = Builder()
+    >>> parser = ElementTree.XMLParser(builder)
+    >>> parser.feed(open("samples/simple-ns.xml", "r").read())
+    pi pi 'data'
+    comment ' comment '
+    start {namespace}root
+    start {namespace}element
+    end {namespace}element
+    start {namespace}element
+    end {namespace}element
+    start {namespace}empty-element
+    end {namespace}empty-element
+    end {namespace}root
+
     """
 
 def getchildren():
@@ -596,7 +629,36 @@ def entity():
 
     """
 
+#
+# reported bugs
+
+class ExceptionFile:
+    def read(self, x):
+        raise IOError
+
+def xmltoolkit60():
+    """
+    Handle crash in stream source.
+    >>> tree = ElementTree.parse(ExceptionFile())
+    Traceback (most recent call last):
+    IOError
+    """
+
+def xmltoolkit61(encoding):
+    """
+    Handle non-standard encodings.
+    >>> xmltoolkit61("ascii")
+    >>> xmltoolkit61("us-ascii")
+    >>> xmltoolkit61("iso-8859-1")
+    >>> xmltoolkit61("iso-8859-15")
+    >>> xmltoolkit61("cp437")
+    >>> xmltoolkit61("mac-roman")
+    """
+    ElementTree.XML(
+        "<?xml version='1.0' encoding='%s'?><xml />" % encoding
+        )
+
 if __name__ == "__main__":
-    import doctest, selftest
-    failed, tested = doctest.testmod(selftest)
+    import doctest
+    failed, tested = doctest.testmod(__import__(__name__))
     print tested - failed, "tests ok."
